@@ -30,11 +30,13 @@ class Block(object):
         "slot",
     )
 
-    def __init__(self, slot, name, drop=None, replace=0, ratio=1,
-            quantity=1, dim=16, breakable=True, orientation=None):
+    def __init__(self, slot, name, secondary=0, drop=None, replace=0, ratio=1,
+        quantity=1, dim=16, breakable=True, orientation=None):
         """
         :param int slot: The index of this block. Must be globally unique.
         :param str name: A common name for this block.
+        :param int secondary: The metadata/damage/secondary attribute for this
+            block. Defaults to zero.
         :param int drop: The type of block that should be dropped when an
             instance of this block is destroyed. Defaults to the slot value, to
             drop instances of this same type of block. To indicate that this
@@ -58,8 +60,7 @@ class Block(object):
         self.slot = slot
         self.name = name
 
-        # XXX
-        self.key = (self.slot, 0)
+        self.key = (self.slot, secondary)
 
         if drop is None:
             self.drop = slot
@@ -142,16 +143,16 @@ class Item(object):
         "slot",
     )
 
-    def __init__(self, slot, name):
+    def __init__(self, slot, name, secondary=0):
 
         self.slot = slot
         self.name = name
 
         # XXX
-        self.key = (self.slot, 0)
+        self.key = (self.slot, secondary)
 
 block_names = [
-    "air",
+    "air", # 0x0
     "stone",
     "grass",
     "dirt",
@@ -167,7 +168,7 @@ block_names = [
     "gravel",
     "gold-ore",
     "iron-ore",
-    "coal-ore",
+    "coal-ore", # 0x10
     "log",
     "leaves",
     "sponge",
@@ -177,17 +178,17 @@ block_names = [
     "dispenser",
     "sandstone",
     "note-block",
-    "aqua-cloth",
-    "cyan-cloth",
-    "blue-cloth",
-    "purple-cloth",
-    "indigo-cloth",
-    "violet-cloth",
-    "magenta-cloth",
-    "pink-cloth",
-    "black-cloth",
-    "grey-cloth",
+    "bed",
+    "powered-rail",
+    "detector-rail",
+    "",
+    "spider-web",
+    "",
+    "", # 0x20
+    "",
+    "",
     "wool",
+    "",
     "flower",
     "rose",
     "brown-mushroom",
@@ -199,7 +200,7 @@ block_names = [
     "brick",
     "tnt",
     "bookshelf",
-    "mossy-cobblestone",
+    "mossy-cobblestone", # 0x30
     "obsidian",
     "torch",
     "fire",
@@ -215,7 +216,7 @@ block_names = [
     "furnace",
     "burning-furnace",
     "signpost",
-    "wooden-door",
+    "wooden-door", # 0x40
     "ladder",
     "tracks",
     "stone-stairs",
@@ -231,7 +232,7 @@ block_names = [
     "stone-button",
     "snow",
     "ice",
-    "snow-block",
+    "snow-block", # 0x50
     "cactus",
     "clay",
     "sugar-cane",
@@ -345,7 +346,7 @@ item_names = [
     "glowstone-dust",
     "raw-fish",
     "cooked-fish",
-    "ink-sack",
+    "dye",
     "bone",
     "sugar",
     "cake",
@@ -359,6 +360,74 @@ special_item_names = [
     "green-music-disc",
 ]
 
+dye_names = [
+    "ink-sac",
+    "red-dye",
+    "green-dye",
+    "cocoa-beans",
+    "lapis-lazuli",
+    "purple-dye",
+    "cyan-dye",
+    "light-gray-dye",
+    "gray-dye",
+    "pink-dye",
+    "lime-dye",
+    "yellow-dye",
+    "light-blue-dye",
+    "magenta-dye",
+    "orange-dye",
+    "bone-meal",
+]
+
+wool_names = [
+    "white-wool",
+    "orange-wool",
+    "magenta-wool",
+    "light-blue-wool",
+    "yellow-wool",
+    "light-green-wool",
+    "pink-wool",
+    "gray-wool",
+    "light-gray-wool",
+    "cyan-wool",
+    "purple-wool",
+    "blue-wool",
+    "brown-wool",
+    "dark-green-wool",
+    "red-wool",
+    "black-wool",
+]
+
+sapling_names = [
+    "normal-sapling",
+    "pine-sapling",
+    "birch-sapling",
+]
+
+log_names = [
+    "normal-log",
+    "pine-log",
+    "birch-log",
+]
+
+leave_names = [
+    "normal-leave",
+    "pine-leave",
+    "birch-leave",
+]
+
+coal_names = [
+    "normal-coal",
+    "charcoal",
+]
+
+step_names = [
+    "stone-step",
+    "sandstone-step",
+    "wooden-step",
+    "cobblestone-step",
+]
+
 drops = {}
 
 # Block -> block drops.
@@ -482,7 +551,7 @@ This dictionary can be indexed by slot number or block name.
 """
 
 for i, name in enumerate(block_names):
-    if name in blocks:
+    if not name or name in blocks:
         continue
 
     kwargs = {}
@@ -510,6 +579,35 @@ for i, name in enumerate(special_item_names):
     items[i] = item
     items[name] = item
 
+_secondary_items = {
+    items["coal"]: coal_names,
+    items["dye"]: dye_names,
+}
+
+for base_item, names in _secondary_items.iteritems():
+    for i, name in enumerate(names):
+        kwargs = {}
+        item = Item(base_item.slot, name, i, **kwargs)
+        items[name] = item
+
+_secondary_blocks = {
+    blocks["leaves"]: leave_names,
+    blocks["log"]: log_names,
+    blocks["sapling"]: sapling_names,
+    blocks["step"]: step_names,
+    blocks["wool"]: wool_names,
+}
+
+for base_block, names in _secondary_blocks.iteritems():
+    for i, name in enumerate(names):
+        kwargs = {}
+        kwargs["drop"] = base_block.drop
+        kwargs["breakable"] = base_block.breakable
+        kwargs["dim"] = base_block.dim
+
+        block = Block(base_block.slot, name, i, **kwargs)
+        _add_block(block)
+
 glowing_blocks = {
     blocks["torch"].slot: 14,
     blocks["lightstone"].slot: 15,
