@@ -1,5 +1,6 @@
 from twisted.plugin import IPlugin
 from twisted.python.components import registerAdapter
+from twisted.web.resource import IResource
 from zope.interface import implements, invariant, Attribute
 
 class InvariantException(Exception):
@@ -390,3 +391,25 @@ class IUseHook(ISortedPlugin):
     targets = Attribute("""
         List of entity names this plugin wants to be called for.
         """)
+
+class IAutomaton(IBravoPlugin):
+    """
+    An automaton.
+
+    Automatons are given blocks from chunks which interest them, and may do
+    processing on those blocks.
+    """
+
+    blocks = Attribute("""
+        List of blocks which this automaton is interested in.
+        """)
+
+    def feed(factory, coordinates):
+        """
+        Provide this automaton with block coordinates to handle later.
+        """
+
+class IWorldResource(IBravoPlugin, IResource):
+    """
+    Interface for a world specific web resource.
+    """
