@@ -9,7 +9,7 @@ from bravo.ibravo import IChatCommand, IConsoleCommand, ISeason
 from bravo.plugin import retrieve_plugins, retrieve_named_plugins
 from bravo.plugin import PluginException
 from bravo.packets.beta import make_packet
-from bravo.utilities import split_time
+from bravo.utilities.temporal import split_time
 
 def parse_player(factory, name):
     if name in factory.protocols:
@@ -133,36 +133,54 @@ class Time(object):
     def dispatch(self, factory):
         hours, minutes = split_time(factory.time)
 
-        # Find seasons, and figure out which season we're in.
+        # Check if the world has seasons enabled
         seasons = retrieve_plugins(ISeason).values()
-        seasons.sort(reverse=True, key=lambda season: season.day)
         if seasons:
-            if all(s.day > factory.day for s in seasons):
-                # We are too close to the beginning of the year; grab the last
-                # season of "last" year.
-                date = "%d (%d %s)" % (factory.day,
-                    factory.day + 360 - seasons[0].day, seasons[0].name)
-            else:
-                # Grab the closest season, Price-is-Right-style.
-                season = (s for s in seasons if s.day <= factory.day).next()
-                date = "%d (%d %s)" % (factory.day, factory.day - season.day,
+            season = factory.world.season
+            day_of_season = factory.day - season.day
+            while day_of_season < 0:
+                day_of_season += 360
+            date = "{0} ({1} {2})".format(factory.day, day_of_season,
                     season.name)
         else:
             date = "%d" % factory.day
         yield "%02d:%02d, %s" % (hours, minutes, date)
 
     def chat_command(self, factory, username, parameters):
-        for i in self.dispatch(factory):
-            yield i
+        if len(parameters) >= 1:
+            # Set the time
+            time = parameters[0]
+            if time == 'sunset':
+                time = 12000
+            elif time == 'sunrise':
+                time = 24000
+            elif ':' in time:
+                # Interpret it as a real-world esque time (24hr clock)
+                hours, minutes = time.split(':')
+                hours, minutes = int(hours), int(minutes)
+                # 24000 ticks / day = 1000 ticks / hour ~= 16.6 ticks / minute
+                time = (hours * 1000) + (minutes * 50 / 3)
+                time -= 6000 # to account for 24000 being high noon in minecraft.
+
+            if len(parameters) >= 2:
+                factory.day = int(parameters[1])
+
+            factory.time = int(time)
+            factory.update_time()
+            factory.update_season()
+            # Update the time for the clients
+            factory.broadcast_time()
+
+        # Tell the user the current time.
+        return self.dispatch(factory)
 
     def console_command(self, factory, parameters):
-        for i in self.dispatch(factory):
-            yield i
+        return self.dispatch(factory)
 
     name = "time"
     aliases = ("date",)
-    usage = ""
-    info = "Gives the current in-game time and date"
+    usage = "[time] [day]"
+    info = "Gives the current in-game time and date, or changes it."
 
 class Say(object):
 
