@@ -21,7 +21,7 @@ from bravo.location import Location
 from bravo.nbt import NBTFile
 from bravo.nbt import TAG_Compound, TAG_List, TAG_Byte_Array, TAG_String
 from bravo.nbt import TAG_Double, TAG_Long, TAG_Short, TAG_Int, TAG_Byte
-from bravo.utilities import unpack_nibbles, pack_nibbles
+from bravo.utilities.bits import unpack_nibbles, pack_nibbles
 
 # Due to technical limitations in the way Twisted discovers plugins, here is
 # how this file works:
@@ -414,6 +414,7 @@ class Alpha(object):
 
         for item in tag.tags:
             slot = item["Slot"].value
+            slot = inventory.decode_slot(slot)
             items[slot] = Slot(item["id"].value,
                 item["Damage"].value, item["Count"].value)
 
@@ -428,10 +429,11 @@ class Alpha(object):
             if item is not None:
                 d = TAG_Compound()
                 id, damage, count = item
+                slot = inventory.encode_slot(i)
                 d["id"] = TAG_Short(id)
                 d["Damage"] = TAG_Short(damage)
                 d["Count"] = TAG_Byte(count)
-                d["Slot"] = TAG_Byte(i)
+                d["Slot"] = TAG_Byte(slot)
                 tag.tags.append(d)
 
         return tag
@@ -630,7 +632,7 @@ class Beta(Alpha):
         handle = fp.open("r")
         handle.seek(position * 4096)
         data = handle.read(pages * 4096)
-        length = unpack(">L", data[:4])[0]
+        length = unpack(">L", data[:4])[0] - 1
         version = ord(data[4])
 
         data = data[5:length+5]
@@ -675,7 +677,7 @@ class Beta(Alpha):
             position, pages = 0, 0
 
         # Pack up the data, all ready to go.
-        data = "%s\x02%s" % (pack(">L", len(data)), data)
+        data = "%s\x02%s" % (pack(">L", len(data) + 1), data)
         needed_pages = (len(data) + 4095) // 4096
 
         handle = fp.open("r+")
