@@ -1,8 +1,8 @@
 from __future__ import division
 
 from itertools import product
-from math import copysign, cos, pi, sin, sqrt
-from random import choice, random
+from math import cos, pi, sin, sqrt
+from random import choice, random, randint
 
 from twisted.internet.defer import inlineCallbacks, returnValue
 from zope.interface import Interface, implements
@@ -11,6 +11,19 @@ from bravo.blocks import blocks
 
 PHI = (sqrt(5) - 1) * 0.5
 IPHI = (sqrt(5) + 1) * 0.5
+"""
+Phi and inverse phi constants.
+"""
+
+# add lights in the middle of foliage clusters
+# for those huge trees that get so dark underneath
+# or for enchanted forests that should glow and stuff
+# Only works if SHAPE is "round" or "cone" or "procedural"
+# 0 makes just normal trees
+# 1 adds one light inside the foliage clusters for a bit of light
+# 2 adds two lights around the base of each cluster, for more light
+# 4 adds lights all around the base of each cluster for lots of light
+LIGHTTREE = 0
 
 @inlineCallbacks
 def dist_to_mat(cord, vec, matidxlist, world, invert=False, limit=None):
@@ -65,15 +78,29 @@ class ITree(Interface):
         """
 
 class Tree(object):
-    """Set up the interface for tree objects.  Designed for subclassing.
+    """
+    Set up the interface for tree objects.  Designed for subclassing.
     """
 
     implements(ITree)
 
-    def __init__(self, height=1, pos=(0, 0, 0)):
-        self.height = height
+    def __init__(self, pos, height=None):
+        if height is None:
+            self.height = randint(4, 7)
+        else:
+            self.height = height
+
         self.pos = pos
 
+    def prepare(self, world):
+        pass
+
+    def make_trunk(self, world):
+        pass
+
+    def make_foliage(self, world):
+        pass
+
 class StickTree(Tree):
     """
     A large stick or log.
@@ -84,7 +111,7 @@ class StickTree(Tree):
     def make_trunk(self, world):
         x, y, z = self.pos
         for i in xrange(self.height):
-            world.set_block((x, y, z), blocks["wood"].slot)
+            world.set_block((x, y, z), blocks["log"].slot)
             y += 1
 
 class NormalTree(StickTree):
@@ -105,13 +132,14 @@ class NormalTree(StickTree):
             else:
                 rad = 2
             for xoff, zoff in product(xrange(-rad, rad + 1), repeat=2):
-                if random() > PHI and abs(xoff) == abs(zoff) == rad:
+                if (random() > PHI and abs(xoff) == abs(zoff) == rad or
+                       xoff == zoff == 0):
                     continue
 
                 x = self.pos[0] + xoff
                 z = self.pos[2] + zoff
 
-                world.set_block((x, y, z), blocks["wood"].slot)
+                world.set_block((x, y, z), blocks["leaves"].slot)
 
 class BambooTree(StickTree):
     """
@@ -129,7 +157,7 @@ class BambooTree(StickTree):
                 zoff = choice([-1, 1])
                 x = self.pos[0] + xoff
                 z = self.pos[2] + zoff
-                world.set_block((x, y, z), blocks["wood"].slot)
+                world.set_block((x, y, z), blocks["leaves"].slot)
 
 class PalmTree(StickTree):
     """
@@ -145,7 +173,7 @@ class PalmTree(StickTree):
             if abs(xoff) == abs(zoff):
                 x = self.pos[0] + xoff
                 z = self.pos[2] + zoff
-                world.set_block((x, y, z), blocks["wood"].slot)
+                world.set_block((x, y, z), blocks["leaves"].slot)
 
 class ProceduralTree(Tree):
     """
@@ -244,9 +272,11 @@ class ProceduralTree(Tree):
         # secidx1 and secidx2 are the remaining indices out of [0,1,2].
         secidx1 = (primidx - 1) % 3
         secidx2 = (1 + primidx) % 3
+
         # primsign is the digit 1 or -1 depending on whether the limb is headed
         # along the positive or negative primidx axis.
-        primsign = copysign(1, delta[primidx])
+        primsign = cmp(delta[primidx], 0) or 1
+
         # secdelta1 and ...2 are the amount the associated values change
         # for every step along the prime axis.
         secdelta1 = delta[secidx1]
@@ -280,9 +310,9 @@ class ProceduralTree(Tree):
 
         foliage_coords = self.foliage_cords
         for coord in foliage_coords:
-            self.foliagecluster(coord,world)
+            self.foliage_cluster(coord,world)
         for x, y, z in foliage_coords:
-            world.set_block((x, y, z), blocks["wood"].slot)
+            world.set_block((x, y, z), blocks["log"].slot)
             if LIGHTTREE == 1:
                 world.set_block((x, y + 1, z), blocks["lightstone"].slot)
             elif LIGHTTREE in [2,4]:
@@ -340,7 +370,169 @@ class ProceduralTree(Tree):
                           treeposition[2]+rndz]
             endsize = 1.0
             self.taperedcylinder(startcoord, coord, startsize, endsize, world,
-                blocks["wood"].slot)
+                blocks["log"].slot)
+
+    def make_trunk(self, world):
+        """
+        Make the trunk, roots, buttresses, branches, etc.
+        """
+
+        # In this method, x and z are the position of the trunk.
+        x, starty, z = self.pos
+        midy = starty + int(self.trunkheight / (PHI + 1))
+        topy = starty + int(self.trunkheight + 0.5)
+        end_size_factor = self.trunkheight / self.height
+        endrad = max(self.trunkradius * (1 - end_size_factor), 1)
+        midrad = max(self.trunkradius * (1 - end_size_factor * .5), endrad)
+
+        # Make the lower and upper sections of the trunk.
+        self.taperedcylinder([x,starty,z], [x,midy,z], self.trunkradius,
+            midrad, world, blocks["log"].slot)
+        self.taperedcylinder([x,midy,z], [x,topy,z], midrad, endrad, world,
+            blocks["log"].slot)
+
+        #Make the branches.
+        self.make_branches(world)
+
+    def prepare(self, world):
+        """
+        Initialize the internal values for the Tree object.
+
+        Primarily, sets up the foliage cluster locations.
+        """
+
+        treeposition = self.pos
+        self.trunkradius = PHI * sqrt(self.height)
+        if self.trunkradius < 1:
+            self.trunkradius = 1
+        self.trunkheight = self.height
+        yend = int(treeposition[1] + self.height)
+        self.branchdensity = 1.0
+        foliage_coords = []
+        ystart = treeposition[1]
+        num_of_clusters_per_y = int(1.5 + (self.height / 19)**2)
+        if num_of_clusters_per_y < 1:
+            num_of_clusters_per_y = 1
+        # make sure we don't spend too much time off the top of the map
+        if yend > 127:
+            yend = 127
+        if ystart > 127:
+            ystart = 127
+        for y in xrange(yend, ystart, -1):
+            for i in xrange(num_of_clusters_per_y):
+                shapefac = self.shapefunc(y - ystart)
+                if shapefac is None:
+                    continue
+                r = (sqrt(random()) + .328)*shapefac
+
+                theta = random()*2*pi
+                x = int(r*sin(theta)) + treeposition[0]
+                z = int(r*cos(theta)) + treeposition[2]
+
+                foliage_coords += [[x,y,z]]
+
+        self.foliage_cords = foliage_coords
+
+
+class RoundTree(ProceduralTree):
+    """
+    A rounded deciduous tree.
+    """
+
+    branchslope = 1 / (PHI + 1)
+    foliage_shape = [2, 3, 3, 2.5, 1.6]
+
+    def prepare(self, world):
+        ProceduralTree.prepare(self, world)
+        self.trunkradius *= 0.8
+        self.trunkheight *= 0.7
+
+    def shapefunc(self, y):
+        twigs = ProceduralTree.shapefunc(self, y)
+        if twigs is not None:
+            return twigs
+        if y < self.height * (.282 + .1 * sqrt(random())) :
+            return None
+        radius = self.height / 2
+        adj = self.height / 2 - y
+        if adj == 0 :
+            dist = radius
+        elif abs(adj) >= radius:
+            dist = 0
+        else:
+            dist = sqrt((radius**2) - (adj**2))
+        dist *= PHI
+        return dist
+
+
+class ConeTree(ProceduralTree):
+    """
+    A conifer.
+    """
+
+    branchslope = 0.15
+    foliage_shape = [3, 2.6, 2, 1]
+
+    def prepare(self, world):
+        ProceduralTree.prepare(self, world)
+        self.trunkradius *= 0.5
+
+    def shapefunc(self, y):
+        twigs = ProceduralTree.shapefunc(self, y)
+        if twigs is not None:
+            return twigs
+        if y < self.height * (.25 + .05 * sqrt(random())):
+            return None
+
+        # Radius.
+        return max((self.height - y) / (PHI + 1), 0)
+
+class RainforestTree(ProceduralTree):
+    """
+    A big rainforest tree.
+    """
+
+    branchslope = 1
+    foliage_shape = [3.4, 2.6]
+
+    def prepare(self, world):
+        ProceduralTree.prepare(self, world)
+        self.trunkradius /= PHI + 1
+        self.trunkheight *= .9
+
+    def shapefunc(self, y):
+        # Bottom 4/5 of the tree is probably branch-free.
+        if y < self.height * 0.8:
+            if self.edgeheight < self.height:
+                twigs = ProceduralTree.shapefunc(self,y)
+                if twigs is not None and random() < 0.07:
+                    return twigs
+            return None
+        else:
+            width = self.height * 1 / (IPHI + 1)
+            topdist = (self.height - y) / (self.height * 0.2)
+            dist = width * (PHI + topdist) * (PHI + random()) * 1 / (IPHI + 1)
+            return dist
+
+class MangroveTree(RoundTree):
+    """
+    A mangrove tree.
+
+    Like the round deciduous tree, but bigger, taller, and generally more
+    awesome.
+    """
+
+    branchslope = 1
+
+    def prepare(self, world):
+        RoundTree.prepare(self, world)
+        self.trunkradius *= PHI
+
+    def shapefunc(self, y):
+        val = RoundTree.shapefunc(self, y)
+        if val is not None:
+            val *= IPHI
+        return val
 
     @inlineCallbacks
     def make_roots(self, rootbases, world):
@@ -378,75 +570,10 @@ class ProceduralTree(Tree):
             # offset is the distance from the root base to the root tip.
             offset = [startcoord[i]-coord[i] for i in xrange(3)]
             # If this is a mangrove tree, make the roots longer.
-            if SHAPE == "mangrove":
-                offset = [int(val * IPHI - 1.5) for val in offset]
-            endcoord = [startcoord[i]+offset[i] for i in xrange(3)]
+            offset = [int(val * IPHI - 1.5) for val in offset]
             rootstartsize = (rootbaseradius * IPHI * abs(offset[1])/
                              (height * IPHI))
-            if rootstartsize < 1.0:
-                rootstartsize = 1.0
-            endsize = 1.0
-            # If ROOTS is set to "tostone" or "hanging" we need to check
-            # along the distance for collision with existing materials.
-            if ROOTS in ["tostone","hanging"]:
-                offlength = sqrt(float(offset[0])**2 +
-                                 float(offset[1])**2 +
-                                 float(offset[2])**2)
-                if offlength < 1:
-                    continue
-                rootmid = endsize
-                # vec is a unit vector along the direction of the root.
-                vec = [offset[i]/offlength for i in xrange(3)]
-                if ROOTS == "tostone":
-                    searchindex = STOPSROOTS
-                elif ROOTS == "hanging":
-                    searchindex = [0]
-                # startdist is how many steps to travel before starting to
-                # search for the material.  It is used to ensure that large
-                # roots will go some distance before changing directions
-                # or stopping.
-                startdist = int(random()*6*sqrt(rootstartsize) + 2.8)
-                # searchstart is the coordinate where the search should begin.
-                searchstart = [startdist * v + sc
-                    for v, sc in zip(vec, startcoord)]
-                # dist stores how far the search went (including searchstart)
-                # before encountering the expected material.
-                dist = startdist
-                dist += yield dist_to_mat(searchstart, vec, searchindex,
-                    world, limit=offlength)
-
-                # If the distance to the material is less than the length
-                # of the root, change the end point of the root to where
-                # the search found the material.
-                if dist < offlength:
-                    # rootmid is the size of the crossection at endcoord.
-                    rootmid +=  (rootstartsize -
-                                         endsize)*(1-dist/offlength)
-                    # endcoord is the midpoint for hanging roots,
-                    # and the endpoint for roots stopped by stone.
-                    endcoord = [startcoord[i]+int(vec[i]*dist)
-                                for i in xrange(3)]
-                    if ROOTS == "hanging":
-                        # remaining_dist is how far the root had left
-                        # to go when it was stopped.
-                        remaining_dist = offlength - dist
-                        # Initialize bottomcord to the stopping point of
-                        # the root, and then hang straight down
-                        # a distance of remaining_dist.
-                        bottomcord = endcoord[:]
-                        bottomcord[1] += -int(remaining_dist)
-                        # Make the hanging part of the hanging root.
-                        self.taperedcylinder(endcoord, bottomcord, rootmid,
-                            endsize, world, blocks["wood"].slot)
-
-                # make the beginning part of hanging or "tostone" roots
-                self.taperedcylinder(startcoord, endcoord, rootstartsize,
-                    rootmid, world, blocks["wood"].slot)
-
-            # If you aren't searching for stone or air, just make the root.
-            else:
-                self.taperedcylinder(startcoord, endcoord, rootstartsize,
-                    endsize, world, blocks["wood"].slot)
+            rootstartsize = max(rootstartsize, 1.0)
 
     def make_trunk(self, world):
         """
@@ -466,244 +593,44 @@ class ProceduralTree(Tree):
         end_size_factor = trunkheight/height
         endrad = max(trunkradius * (1 - end_size_factor), 1)
         midrad = max(trunkradius * (1 - end_size_factor * .5), endrad)
-        # Make the root buttresses, if indicated
-        if ROOTBUTTRESSES or SHAPE == "mangrove":
-            # The start radius of the trunk should be a little smaller if we
-            # are using root buttresses.
-            startrad = trunkradius * .8
-            # rootbases is used later in self.makeroots(...) as
-            # starting locations for the roots.
-            rootbases = [[x,z,startrad]]
-            buttress_radius = trunkradius * 0.382
-            # posradius is how far the root buttresses should be offset
-            # from the trunk.
-            posradius = trunkradius
-            # In mangroves, the root buttresses are much more extended.
-            if SHAPE == "mangrove":
-                posradius = posradius * (IPHI + 1)
-            num_of_buttresses = int(sqrt(trunkradius) + 3.5)
-            for i in xrange(num_of_buttresses):
-                rndang = random()*2*pi
-                thisposradius = posradius * (0.9 + random()*.2)
-                # thisx and thisz are the x and z position for the base of
-                # the root buttress.
-                thisx = x + int(thisposradius * sin(rndang))
-                thisz = z + int(thisposradius * cos(rndang))
-                # thisbuttressradius is the radius of the buttress.
-                # Currently, root buttresses do not taper.
-                thisbuttressradius = max(buttress_radius * (PHI + random()),
-                    1)
-                # Make the root buttress.
-                self.taperedcylinder([thisx, starty, thisz], [x, midy, z],
-                    thisbuttressradius, thisbuttressradius, world,
-                    blocks["wood"].slot)
-                # Add this root buttress as a possible location at
-                # which roots can spawn.
-                rootbases += [[thisx,thisz,thisbuttressradius]]
-        else:
-            # If root buttresses are turned off, set the trunk radius
-            # to normal size.
-            startrad = trunkradius
-            rootbases = [[x,z,startrad]]
+
+        # The start radius of the trunk should be a little smaller if we
+        # are using root buttresses.
+        startrad = trunkradius * .8
+        # rootbases is used later in self.makeroots(...) as
+        # starting locations for the roots.
+        rootbases = [[x,z,startrad]]
+        buttress_radius = trunkradius * 0.382
+        # posradius is how far the root buttresses should be offset
+        # from the trunk.
+        posradius = trunkradius
+        # In mangroves, the root buttresses are much more extended.
+        posradius = posradius * (IPHI + 1)
+        num_of_buttresses = int(sqrt(trunkradius) + 3.5)
+        for i in xrange(num_of_buttresses):
+            rndang = random()*2*pi
+            thisposradius = posradius * (0.9 + random()*.2)
+            # thisx and thisz are the x and z position for the base of
+            # the root buttress.
+            thisx = x + int(thisposradius * sin(rndang))
+            thisz = z + int(thisposradius * cos(rndang))
+            # thisbuttressradius is the radius of the buttress.
+            # Currently, root buttresses do not taper.
+            thisbuttressradius = max(buttress_radius * (PHI + random()),
+                1)
+            # Make the root buttress.
+            self.taperedcylinder([thisx, starty, thisz], [x, midy, z],
+                thisbuttressradius, thisbuttressradius, world,
+                blocks["log"].slot)
+            # Add this root buttress as a possible location at
+            # which roots can spawn.
+            rootbases += [[thisx,thisz,thisbuttressradius]]
+
         # Make the lower and upper sections of the trunk.
         self.taperedcylinder([x,starty,z], [x,midy,z], startrad, midrad,
-            world, blocks["wood"].slot)
+            world, blocks["log"].slot)
         self.taperedcylinder([x,midy,z], [x,topy,z], midrad, endrad, world,
-            blocks["wood"].slot)
+            blocks["log"].slot)
+
         #Make the branches
-        self.makebranches(world)
-        #Make the roots, if indicated.
-        if ROOTS in ["yes","tostone","hanging"]:
-            self.makeroots(rootbases, world)
-        # Hollow the trunk, if specified
-        # check to make sure that the trunk is large enough to be hollow
-        if trunkradius > 2 and HOLLOWTRUNK:
-            # wall thickness is actually the double the wall thickness
-            # it is a diameter difference, not a radius difference.
-            wall_thickness = max(1 + trunkradius * 0.1 * random(), 1.3)
-            base_radius = max(trunkradius - wall_thickness, 1)
-            mid_radius = midrad - wall_thickness
-            top_radius = endrad - wall_thickness
-            # the starting x and y can be offset by up to the wall thickness.
-            base_offset = int(wall_thickness)
-            x_choices = range(x - base_offset, x + base_offset + 1)
-            z_choices = range(z - base_offset, z + base_offset + 1)
-            start_x = choice(x_choices)
-            start_z = choice(z_choices)
-            self.taperedcylinder([start_x, starty, start_z], [x, midy, z],
-                base_radius, mid_radius, world, blocks["air"].slot)
-            hollow_top_y = int(topy + trunkradius + 1.5)
-            self.taperedcylinder([x, midy, z], [x, hollow_top_y, z],
-                mid_radius, top_radius, world, blocks["air"].slot)
-
-    @inlineCallbacks
-    def prepare(self,world):
-        """Initialize the internal values for the Tree object.
-
-        Primarily, sets up the foliage cluster locations.
-        """
-        treeposition = self.pos
-        self.trunkradius = PHI * sqrt(self.height * TRUNKTHICKNESS)
-        if self.trunkradius < 1:
-            self.trunkradius = 1
-        if BROKENTRUNK:
-            self.trunkheight = self.height * ( .3 + random() * .4 )
-            yend = int(treeposition[1] + self.trunkheight + .5)
-        else:
-            self.trunkheight = self.height
-            yend = int(treeposition[1] + self.height)
-        self.branchdensity = BRANCHDENSITY / FOLIAGEDENSITY
-        topy = treeposition[1]+int(self.trunkheight + 0.5)
-        foliage_coords = []
-        ystart = treeposition[1]
-        num_of_clusters_per_y = int(1.5 + (FOLIAGEDENSITY *
-                                           self.height / 19)**2)
-        if num_of_clusters_per_y < 1:
-            num_of_clusters_per_y = 1
-        # make sure we don't spend too much time off the top of the map
-        if yend > 127:
-            yend = 127
-        if ystart > 127:
-            ystart = 127
-        for y in xrange(yend, ystart, -1):
-            for i in xrange(num_of_clusters_per_y):
-                shapefac = self.shapefunc(y - ystart)
-                if shapefac is None:
-                    continue
-                r = (sqrt(random()) + .328)*shapefac
-
-                theta = random()*2*pi
-                x = int(r*sin(theta)) + treeposition[0]
-                z = int(r*cos(theta)) + treeposition[2]
-                # if there are values to search in STOPSBRANCHES
-                # then check to see if this cluster is blocked
-                # by stuff, like dirt or rock, or whatever
-                if len(STOPSBRANCHES):
-                    dist = (sqrt(float(x-treeposition[0])**2 +
-                                float(z-treeposition[2])**2))
-                    slope = self.branchslope
-                    if y - dist*slope > topy:
-                        # the top of the tree
-                        starty = topy
-                    else:
-                        starty = y-dist*slope
-                    # the start position of the search
-                    start = [treeposition[0], starty, treeposition[2]]
-                    offset = [x - treeposition[0],
-                              y - starty,
-                              z - treeposition[2]]
-                    offlength = sqrt(offset[0]**2 + offset[1]**2 + offset[2]**2)
-                    # if the branch is longer than... nothing
-                    if offlength > 0:
-                        # unit vector for the search
-                        vec = [i / offlength for i in offset]
-                        mat_dist = yield dist_to_mat(start, vec,
-                            STOPSBRANCHES, world, limit=offlength)
-                        # after all that, if you find something, don't add
-                        # this coordinate to the list
-                        if mat_dist < offlength:
-                            continue
-                foliage_coords += [[x,y,z]]
-
-        self.foliage_cords = foliage_coords
-
-
-class RoundTree(ProceduralTree):
-    """
-    A rounded deciduous tree.
-    """
-
-    branchslope = 1 / (PHI + 1)
-    foliage_shape = [2, 3, 3, 2.5, 1.6]
-
-    def prepare(self, world):
-        ProceduralTree.prepare(self, world)
-        self.trunkradius *= 0.8
-        self.trunkheight *= TRUNKHEIGHT
-
-    def shapefunc(self, y):
-        twigs = ProceduralTree.shapefunc(self, y)
-        if twigs is not None:
-            return twigs
-        if y < self.height * (.282 + .1 * sqrt(random())) :
-            return None
-        radius = self.height / 2
-        adj = self.height / 2 - y
-        if adj == 0 :
-            dist = radius
-        elif abs(adj) >= radius:
-            dist = 0
-        else:
-            dist = sqrt((radius**2) - (adj**2))
-        dist *= PHI
-        return dist
-
-
-class ConeTree(ProceduralTree):
-    """
-    A conifer.
-    """
-
-    branchslope = 0.15
-    foliage_shape = [3, 2.6, 2, 1]
-
-    def prepare(self, world):
-        ProceduralTree.prepare(self, world)
-        self.trunkradius *= 0.5
-
-    def shapefunc(self, y):
-        twigs = ProceduralTree.shapefunc(self, y)
-        if twigs is not None:
-            return twigs
-        if y < self.height * (.25 + .05 * sqrt(random())) :
-            return None
-
-        # Radius.
-        return max((self.height - y) / (PHI + 1), 0)
-
-class RainforestTree(ProceduralTree):
-    """
-    A big rainforest tree.
-    """
-
-    branchslope = 1
-    foliage_shape = [3.4, 2.6]
-
-    def prepare(self, world):
-        ProceduralTree.prepare(self, world)
-        self.trunkradius /= PHI + 1
-        self.trunkheight *= .9
-
-    def shapefunc(self, y):
-        # Bottom 4/5 of the tree is probably branch-free.
-        if y < self.height * 0.8:
-            if EDGEHEIGHT < self.height:
-                twigs = ProceduralTree.shapefunc(self,y)
-                if twigs is not None and random() < 0.07:
-                    return twigs
-            return None
-        else:
-            width = self.height * 1 / (IPHI + 1)
-            topdist = (self.height - y) / (self.height * 0.2)
-            dist = width * (PHI + topdist) * (PHI + random()) * 1 / (IPHI + 1)
-            return dist
-
-class MangroveTree(RoundTree):
-    """
-    A mangrove tree.
-
-    Like the round deciduous tree, but bigger, taller, and generally more
-    awesome.
-    """
-
-    branchslope = 1
-
-    def prepare(self, world):
-        RoundTree.prepare(self, world)
-        self.trunkradius *= PHI
-
-    def shapefunc(self, y):
-        val = RoundTree.shapefunc(self, y)
-        if val is not None:
-            val *= IPHI
-        return val
+        self.make_branches(world)
