@@ -21,9 +21,9 @@ class PhysicsMockFactory(object):
 class TestWater(unittest.TestCase):
 
     def setUp(self):
-        # Using build hook to grab the plugin, but dig hook should work as
-        # well.
-        self.p = bravo.plugin.retrieve_plugins(bravo.ibravo.IBuildHook)
+        # Using dig hook to grab the plugin since the build hook was nuked in
+        # favor of the automaton interface.
+        self.p = bravo.plugin.retrieve_plugins(bravo.ibravo.IDigHook)
 
         if "water" not in self.p:
             raise unittest.SkipTest("Plugin not present")
@@ -105,7 +105,85 @@ class TestWater(unittest.TestCase):
         self.assertEqual(block, bravo.blocks.blocks["water"].slot)
         self.assertEqual(metadata, 0x8)
 
-    test_spring_fall.todo = "Metadata is wrong for some reason"
+    @inlineCallbacks
+    def test_spring_fall_dig(self):
+        """
+        Destroying ground underneath spring should allow water to continue
+        falling downwards.
+        """
+
+        self.w.set_block((0, 1, 0), bravo.blocks.blocks["spring"].slot)
+        self.w.set_block((0, 0, 0), bravo.blocks.blocks["dirt"].slot)
+        self.hook.pending[self.f].add((0, 1, 0))
+
+        # Tight-loop run the hook to equilibrium.
+        while self.hook.pending:
+            self.hook.process()
+
+        #dig away dirt under spring
+        self.w.destroy((0, 0, 0))
+        self.hook.pending[self.f].add((0, 1, 0))
+
+        while self.hook.pending:
+            self.hook.process()
+
+        block = yield self.w.get_block((0, 0, 0))
+        self.assertEqual(block, bravo.blocks.blocks["water"].slot)
+
+    @inlineCallbacks
+    def test_spring_fall_dig_offset(self):
+        """
+        Destroying ground next to a spring should cause a waterfall effect.
+        """
+
+        self.w.set_block((0, 1, 0), bravo.blocks.blocks["spring"].slot)
+        self.w.set_block((0, 0, 0), bravo.blocks.blocks["dirt"].slot)
+        self.w.set_block((0, 0, 1), bravo.blocks.blocks["dirt"].slot)
+        self.hook.pending[self.f].add((0, 1, 0))
+
+        # Tight-loop run the hook to equilibrium.
+        while self.hook.pending:
+            self.hook.process()
+
+        # Dig away the dirt next to the dirt under the spring, and simulate
+        # the dig hook by adding the block above it.
+        self.w.destroy((0, 0, 1))
+        self.hook.pending[self.f].add((0, 1, 1))
+
+        while self.hook.pending:
+            self.hook.process()
+
+        block = yield self.w.get_block((0, 0, 1))
+        self.assertEqual(block, bravo.blocks.blocks["water"].slot)
+
+    @inlineCallbacks
+    def test_spring_waterfall(self):
+        """
+        Fluid should not spread across existing fluid.
+        """
+
+        self.w.set_block((0, 3, 0), bravo.blocks.blocks["spring"].slot)
+        self.w.set_block((0, 2, 0), bravo.blocks.blocks["dirt"].slot)
+        self.hook.pending[self.f].add((0, 1, 0))
+
+        # Tight-loop run the hook to equilibrium.
+        while self.hook.pending:
+            self.hook.process()
+
+        #dig away dirt and add known spring and fluid blocks
+        self.w.destroy((0, 2, 0))
+        self.hook.pending[self.f].add((0, 2, 1))
+        self.hook.pending[self.f].add((0, 2, -1))
+        self.hook.pending[self.f].add((0, 3, 0))
+        self.hook.pending[self.f].add((1, 2, 0))
+        self.hook.pending[self.f].add((-1, 2, 0))
+
+        while self.hook.pending:
+            self.hook.process()
+
+        block = yield self.w.get_block((0, 1, 2))
+        self.assertEqual(block, bravo.blocks.blocks["air"].slot)
+
 
     @inlineCallbacks
     def test_obstacle(self):
