@@ -1,9 +1,8 @@
 from twisted.trial import unittest
 
 import bravo.blocks
-import bravo.inventory
 
-from bravo.inventory import Slot
+from bravo.inventory import Equipment, Inventory, Slot, Workbench
 
 class TestSlot(unittest.TestCase):
     """
@@ -19,6 +18,15 @@ class TestSlot(unittest.TestCase):
         slot2 = Slot(4, 5, 1)
         self.assertTrue(slot1.holds(slot2))
 
+    def test_holds_secondary(self):
+        """
+        Secondary attributes always matter for .holds.
+        """
+
+        slot1 = Slot(4, 5, 1)
+        slot2 = Slot(4, 6, 1)
+        self.assertFalse(slot1.holds(slot2))
+
 class TestInventoryInternals(unittest.TestCase):
     """
     The Inventory class might be near-useless when not subclassed, but we can
@@ -26,7 +34,7 @@ class TestInventoryInternals(unittest.TestCase):
     """
 
     def setUp(self):
-        self.i = bravo.inventory.Inventory()
+        self.i = Inventory()
 
     def test_trivial(self):
         pass
@@ -37,7 +45,7 @@ class TestInventoryInternals(unittest.TestCase):
 class TestEquipmentInternals(unittest.TestCase):
 
     def setUp(self):
-        self.i = bravo.inventory.Equipment()
+        self.i = Equipment()
 
     def test_trivial(self):
         pass
@@ -164,7 +172,7 @@ class TestEquipmentInternals(unittest.TestCase):
         self.assertEqual(self.i.selected, (2, 0, 40))
         # second stack is untouched
         self.assertEqual(self.i.holdables[1], (2, 0, 30))
-        # select second stack with right click
+        # select second stack with left click
         self.i.select(37)
         # sums up to more than 64 items - fill up the second stack
         self.assertEqual(self.i.holdables[1], (2, 0, 64))
@@ -181,12 +189,24 @@ class TestEquipmentInternals(unittest.TestCase):
         self.assertEqual(self.i.selected, (2, 0, 40))
         # second stack is untouched
         self.assertEqual(self.i.holdables[1], (2, 0, 30))
-        # select second stack with left click
+        # select second stack with right click
         self.i.select(37, True)
-        # sums up to more than 64 items - fill up the second stack
-        self.assertEqual(self.i.holdables[1], (2, 0, 64))
+        # sums up to more than 64 items
+        self.assertEqual(self.i.holdables[1], (2, 0, 31))
         # still hold the left overs
-        self.assertEqual(self.i.selected, (2, 0, 6))
+        self.assertEqual(self.i.selected, (2, 0, 39))
+
+    def test_encoder_ring(self):
+        self.assertEqual(self.i.encode_slot(36), 0)
+
+    def test_encoder_ring_noop(self):
+        self.assertEqual(self.i.encode_slot(20), 20)
+
+    def test_decoder_ring(self):
+        self.assertEqual(self.i.decode_slot(100), 8)
+
+    def test_decoder_ring_noop(self):
+        self.assertEqual(self.i.decode_slot(21), 21)
 
 class TestCraftingWood(unittest.TestCase):
     """
@@ -198,7 +218,7 @@ class TestCraftingWood(unittest.TestCase):
     """
 
     def setUp(self):
-        self.i = bravo.inventory.Equipment()
+        self.i = Equipment()
 
     def test_trivial(self):
         pass
@@ -238,7 +258,7 @@ class TestCraftingSticks(unittest.TestCase):
     """
 
     def setUp(self):
-        self.i = bravo.inventory.Equipment()
+        self.i = Equipment()
 
     def test_trivial(self):
         pass
@@ -280,7 +300,7 @@ class TestCraftingTorches(unittest.TestCase):
     """
 
     def setUp(self):
-        self.i = bravo.inventory.Equipment()
+        self.i = Equipment()
 
     def test_trivial(self):
         pass
@@ -323,7 +343,7 @@ class TestCraftingFurnace(unittest.TestCase):
     """
 
     def setUp(self):
-        self.i = bravo.inventory.Workbench()
+        self.i = Workbench()
 
     def test_trivial(self):
         pass
@@ -361,7 +381,7 @@ class TestCraftingFurnace(unittest.TestCase):
 class TestInventoryIntegration(unittest.TestCase):
 
     def setUp(self):
-        self.i = bravo.inventory.Equipment()
+        self.i = Equipment()
 
     def test_trivial(self):
         pass
@@ -448,6 +468,12 @@ class TestInventoryIntegration(unittest.TestCase):
         self.assertEqual(self.i.selected, (bravo.blocks.items["gold-helmet"].slot, 0, 5))
 
     def test_armor_slots_take_armor_items_only(self):
+        """
+        Confirm that dirt cannot be used as a helmet.
+
+        This is the exact test case from #175.
+        """
+
         self.i.add((bravo.blocks.blocks["dirt"].slot, 0), 10)
         self.i.select(36)
         self.assertFalse(self.i.select(5))
@@ -487,7 +513,7 @@ class TestWorkbenchIntegration(unittest.TestCase):
     """
 
     def setUp(self):
-        self.i = bravo.inventory.Workbench()
+        self.i = Workbench()
 
     def test_trivial(self):
         pass
