@@ -18,7 +18,8 @@ from bravo.errors import SerializerReadException
 from bravo.ibravo import ISerializer, ISerializerFactory
 from bravo.plugin import (retrieve_named_plugins, verify_plugin,
     PluginException)
-from bravo.utilities import fork_deferred, split_coords
+from bravo.utilities.coords import split_coords
+from bravo.utilities.temporal import PendingEvent
 
 def coords_to_chunk(f):
     """
@@ -217,24 +218,6 @@ class World(object):
         self.chunk_cache = d
         self.saving = True
 
-    def populate_chunk(self, chunk):
-        """
-        Recreate data for a chunk.
-
-        This method does arbitrary terrain generation depending on the current
-        plugins, and then regenerates the chunk metadata so that the chunk can
-        be sent to clients.
-
-        A lot of maths may be done by this method, so do not call it unless
-        absolutely necessary, e.g. when the chunk is created for the first
-        time.
-        """
-
-        for stage in self.pipeline:
-            stage.populate(chunk, self.seed)
-
-        chunk.regenerate()
-
     def postprocess_chunk(self, chunk):
         """
         Do a series of final steps to bring a chunk into the world.
@@ -271,7 +254,7 @@ class World(object):
             returnValue(self.dirty_chunk_cache[x, z])
         elif (x, z) in self._pending_chunks:
             # Rig up another Deferred and wrap it up in a to-go box.
-            retval = yield fork_deferred(self._pending_chunks[x, z])
+            retval = yield self._pending_chunks[x, z].deferred()
             returnValue(retval)
 
         chunk = Chunk(x, z)
@@ -292,7 +275,6 @@ class World(object):
                 seed=self.seed,
                 generators=configuration.getlist(self.config_name, "generators")
             )
-            self._pending_chunks[x, z] = d
 
             # Get chunk data into our chunk object.
             def fill_chunk(kwargs):
@@ -310,9 +292,19 @@ class World(object):
                 return chunk
             d.addCallback(fill_chunk)
         else:
-            self.populate_chunk(chunk)
+            # Populate the chunk the slow way. :c
+            for stage in self.pipeline:
+                stage.populate(chunk, self.seed)
+
+            chunk.regenerate()
             d = succeed(chunk)
-            self._pending_chunks[x, z] = d
+
+        # Set up our event and generate our return-value Deferred. It has to
+        # be done early becaues PendingEvents only fire exactly once and it
+        # might fire immediately in certain cases.
+        pe = PendingEvent()
+        retval = pe.deferred()
+        self._pending_chunks[x, z] = pe
 
         def pp(chunk):
             chunk.populated = True
@@ -327,11 +319,16 @@ class World(object):
 
         # Set up callbacks.
         d.addCallback(pp)
+        d.chainDeferred(pe)
 
-        # Multiple people might be subscribed to this pending callback. We're
-        # going to keep it for ourselves and fork off another Deferred for our
-        # caller.
-        retval = yield fork_deferred(d)
+        # Because multiple people might be attached to this callback, we're
+        # going to do something magical here. We will yield a forked version
+        # of our Deferred. This means that we will wait right here, for a
+        # long, long time, before actually returning with the chunk, *but*,
+        # when we actually finish, we'll be ready to return the chunk
+        # immediately. Our caller cannot possibly care because they only see a
+        # Deferred either way.
+        retval = yield retval
         returnValue(retval)
 
     def save_chunk(self, chunk):
